#!/usr/bin/env python
# PROGRAM:
# The code used to execute the similarity computations (3D, 2D, and PPI) 
# performed in Yera JMC 2013
#

__author__="Emmanuel R. Yera"
__date__ ="$May 20, 2013 01:33:53 PM$"

import os,sys,math,commands,utils,re,copy

class Target:
    def __init__(self,name,file,drugs,all_scores):
        """
        Initialize a Target

        """
        self.target_name = name
        self.all_scores = all_scores
        self.drugs_all = []
        self.drugs_all_type = {}
        self.drugs_all_hash = {}
        self.p_values_ppi = ""
        self.p_values_ppi_names = ""
        self.count_primary = 0
        self.count_secondary = 0
        self.drugs_file = file
        self.drugs = drugs
        self.multinomial_prob = [(0.01-0.0),(0.05-0.01),(0.1-0.05),(0.2-0.1),(0.5-0.2),(1.0-0.5)]

        self.parse_target()
	
    def parse_target(self):
        """
        Parse the target file.
        """
        for line in self.drugs_file:
            tokens = line.split(";")
            interactiontype = tokens[3]
            drug_name = tokens[6]
            if self.drugs.has_key(drug_name):
                self.drugs_all.append(drug_name)
                self.drugs_all_hash[drug_name] = tokens[4]
                if tokens[4] == "primary":
                    self.drugs_all_type[tokens[6]] = "primary"
                    self.count_primary = self.count_primary + 1
                elif tokens[4] == "secondary":
                    self.drugs_all_type[tokens[6]] = "secondary"
                    self.count_secondary = self.count_secondary + 1
            else:
                print "not found: ",drug_name


    def print_drugs(self):
        """
        Print the drugs in the target
        """
        print "drugs: ",self.drugs_all

    def print_info(self):
        """
        Print information about this Target
        """
        print self.target_name+" len(drugs_all):" + str(len(self.drugs_all))

    def similarity(self,test_mol):
        """
        Compute the similarity of a test molecule to this specific target by computing 
        the log-odds (S = -log (M / M*) where M is the multinomial prob of observing 
        the discritized p-values and M* is for converse probabilities.
        """
        # compute the similarity between all of our drugs to the test molecule
        [total_count,hist_ppi1,p_values_ppi1] = self.similarity_drugs(test_mol,"are")
        
        # compute the multimodal probability
        prob_ppi_are = self.multi(total_count,hist_ppi1) 

        # reinitialize the variables
        total_count = 0
        # we are now going to compute the are not
        [total_count,hist_ppi2,p_values_ppi2] = self.similarity_drugs(test_mol,"arenot")
        
        # compute the multimodal probability
        prob_ppi_arenot = self.multi(total_count,hist_ppi2) 
        
        odd_ppi = 0
        div_ppi = 0
        
        final_odd = 0
        final_div = 0

        if prob_ppi_arenot == 0 or prob_ppi_are == 0:
            odd_ppi = 0
        else:
            div_ppi = prob_ppi_are / prob_ppi_arenot
            odd_ppi = -1 * math.log10(div_ppi)

        return [odd_ppi,prob_ppi_are,prob_ppi_arenot,div_ppi,hist_ppi1,hist_ppi2,p_values_ppi1,p_values_ppi2]

    def similarity_comb(self,test_mol):
        """
        Compute the log-odds by combining two similarity methods.
        """
        # compute the similarity between all of our drugs to the test molecule
        [total_count,hist_ppi1,p_values_ppi1] = self.similarity_drugs_comb(test_mol,"are")
        
        # compute the multimodal probability
        prob_ppi_are = self.multi(total_count,hist_ppi1) 

        # reinitialize the variables
        total_count = 0
        # we are now going to compute the are not
        [total_count,hist_ppi2,p_values_ppi2] = self.similarity_drugs_comb(test_mol,"arenot")

        # compute the multimodal probability
        prob_ppi_arenot = self.multi(total_count,hist_ppi2) 

        odd_ppi = 0
        div_ppi = 0

        final_odd = 0
        final_div = 0

        if prob_ppi_arenot == 0 or prob_ppi_are == 0:
            odd_ppi = 0
        else:
            div_ppi = prob_ppi_are / prob_ppi_arenot
            odd_ppi = -1 * math.log10(div_ppi)

        return [odd_ppi,prob_ppi_are,prob_ppi_arenot,div_ppi,hist_ppi1,hist_ppi2,p_values_ppi1,p_values_ppi2]
        
    def similarity_comb3(self,test_mol):
        """
        Compute the log-odds by combining three similarity methods.
        """
        # compute the similarity between all of our drugs to the test molecule
        [total_count,hist_ppi1,p_values_ppi1] = self.similarity_drugs_comb3(test_mol,"are")
        
        # compute the multimodal probability
        prob_ppi_are = self.multi(total_count,hist_ppi1) 

        # reinitialize the variables
        total_count = 0
        # we are now going to compute the are not
        [total_count,hist_ppi2,p_values_ppi2] = self.similarity_drugs_comb3(test_mol,"arenot")
        
        # compute the multimodal probability
        prob_ppi_arenot = self.multi(total_count,hist_ppi2) 

        odd_ppi = 0
        div_ppi = 0

        final_odd = 0
        final_div = 0

        if prob_ppi_arenot == 0 or prob_ppi_are == 0:
            odd_ppi = 0
        else:
            div_ppi = prob_ppi_are / prob_ppi_arenot
            odd_ppi = -1 * math.log10(div_ppi)


        return [odd_ppi,prob_ppi_are,prob_ppi_arenot,div_ppi,hist_ppi1,hist_ppi2,p_values_ppi1,p_values_ppi2]

    
    def similarity_drugs(self,test_mol,type):
        """
        Get the similarities and put them in the corresponding 
        bin of the p-value count histogram.
        """
        hist_ppi = [0,0,0,0,0,0]
        self.p_values_ppi = []
        self.p_values_ppi_names = []
        total_count = 0
        
        # get the similarity between all of our drugs to the test molecule
        for mol in self.drugs_all:
            if test_mol != mol:
                key = test_mol+" "+mol
                if self.all_scores.has_key(key):
                    # get the scores
                    score = self.all_scores[key]
                    p_val_ppi = 0.0
                    
                    if type == "are":
                        p_val_ppi = float(score[1])
                    else:
                        p_val_ppi = 1.0 - float(score[1])
                        
                    # hack to get the sim scores out
                    self.p_values_ppi.append(1.0-p_val_ppi)
                    self.p_values_ppi_names.append(mol)
                    
                    # add the new p_values to their histrogram
                    hist_ppi = self.compute_hist(hist_ppi,p_val_ppi) 
                    
                    total_count = total_count + 1
                else:
                    print "sim not found:",key
        
        return [total_count,hist_ppi,self.p_values_ppi]
    
    def similarity_drugs_comb(self,test_mol,type):
        """
        Get the similarities of two similarit computations and put 
        them in the  bin of the p-value count histogram.
        """
        hist_comb = [0,0,0,0,0,0]
        p_values_comb = []
        total_count = 0

        # get the similarity between all of our drugs to the test molecule
        for mol in self.drugs_all:
            if test_mol != mol:
                key = test_mol+" "+mol
                if self.all_scores.has_key(key):
                    # get the scores
                    score = self.all_scores[key]
                    p_val_3d = 0.0
                    p_val_2d = 0.0
                    if type == "are":
                        p_val_3d = float(score[1])
                        p_val_2d = float(score[3])
                    else:
                        p_val_3d = 1.0 - float(score[1])
                        p_val_2d = 1.0 - float(score[3])

                    p_values_comb.append(p_val_3d)
                    p_values_comb.append(p_val_2d)

                    # add the new p_values to their histrogram
                    hist_comb = self.compute_hist(hist_comb,p_val_3d) 
                    hist_comb = self.compute_hist(hist_comb,p_val_2d) 

                    total_count = total_count + 2


        return [total_count,hist_comb,p_values_comb]
    
    def similarity_drugs_comb3(self,test_mol,type):
        """
        Get the similarities of three similarit computations and put 
        them in the  bin of the p-value count histogram.
        """
        hist_comb = [0,0,0,0,0,0]
        p_values_comb = []
        total_count = 0

        # get the similarity between all of our drugs to the test molecule
        for mol in self.drugs_all:
            if test_mol != mol:
                key = test_mol+" "+mol
                if self.all_scores.has_key(key):
                    # get the scores
                    score = self.all_scores[key]
                    p_val_3d = 0.0
                    p_val_2d = 0.0
                    p_val_ppi = 0.0
                    if type == "are":
                        p_val_3d = float(score[1])
                        p_val_2d = float(score[3])
                        p_val_ppi = float(score[5])
                    else:
                        p_val_3d = 1.0 - float(score[1])
                        p_val_2d = 1.0 - float(score[3])
                        p_val_ppi = 1.0 - float(score[5])

                    p_values_comb.append(p_val_3d)
                    p_values_comb.append(p_val_2d)
                    p_values_comb.append(p_val_ppi)

                    # add the new p_values to their histrogram
                    hist_comb = self.compute_hist(hist_comb,p_val_3d) 
                    hist_comb = self.compute_hist(hist_comb,p_val_2d) 
                    hist_comb = self.compute_hist(hist_comb,p_val_ppi) 

                    total_count = total_count + 3

        return [total_count,hist_comb,p_values_comb]
    
    def compute_hist(self,hist,p_val):
        """
        Increment the corresponding bin of the histagram of the p_val
        """

        if p_val >= 0.0 and p_val < 0.01:
            hist[0] = hist[0] + 1
        elif p_val >= 0.01 and p_val < 0.05:
            hist[1] = hist[1] + 1
        elif p_val >= 0.05 and p_val < 0.1:
            hist[2] = hist[2] + 1
        elif p_val >= 0.1 and p_val < 0.2:
            hist[3] = hist[3] + 1
        elif p_val >= 0.2 and p_val < 0.5:
            hist[4] = hist[4] + 1
        else:
            hist[5] = hist[5] + 1
        return hist

    def factorial(self,n):
        """
        Compute n factorial
        """
        result = 1
        for i in xrange(2, n+1):
            result *= i
        return result
    
    def multi(self,total_count,hist):
        """
        Compute the probablities based on the multinomial 
        distribution observing the p-values
        """
        expected_prob_top = self.factorial(total_count)
        expected_prob_bottom = (self.factorial(hist[0])*self.factorial(hist[1])*self.factorial(hist[2])*self.factorial(hist[3])*self.factorial(hist[4])*self.factorial(hist[5]))
        p0 = math.pow(self.multinomial_prob[0],hist[0])
        if p0 == 0:
            p0 = 1.0

        p1 = math.pow(self.multinomial_prob[1],hist[1])
        if p1 == 0:
            p1 = 1.0

        p2 = math.pow(self.multinomial_prob[2],hist[2])
        if p2 == 0:
            p2 = 1.0

        p3 = math.pow(self.multinomial_prob[3],hist[3])
        if p3 == 0:
            p3 = 1.0

        p4 = math.pow(self.multinomial_prob[4],hist[4])
        if p4 == 0:
            p4 = 1.0

        p5 = math.pow(self.multinomial_prob[5],hist[5])
        if p5 == 0:
            p5 = 1.0

        expected_prob_prob = p0*p1*p2*p3*p4*p5


        try:
            expected_prob = (expected_prob_top / expected_prob_bottom) * expected_prob_prob
        except OverflowError:
            expected_prob = 1.1
            
        if expected_prob >= 1.0:
            #print "expected_prob > 1.0: ",expected_prob
            #print "expected_prob_top: ",expected_prob_top
            #print "expected_prob_bottom: ",expected_prob_bottom
            #print "expected_prob_prob: ",expected_prob_prob
            #print "total_count: ",total_count
            #print hist
            expected_prob = 0.0

        return expected_prob
    

def read_drug_ppi(drugs_to_read,ngram_size,file_ext,file_path,common_terms):
    """
    Read the PPI drug vectores from file. 
    
    """
    # read all drugs
    drugs_hash = {}
    for d in drugs_to_read:
        drug1_ngrams = {}
        drug1_ngrams_file = open(file_path+d+file_ext)
        for line1 in drug1_ngrams_file:
            tokens = line1.split(",")
            if len(tokens) == 2:
                term = tokens[0]
                if len(term.split(" ")) <= ngram_size:
                    if not common_terms.has_key(term):
                        weight = tokens[1]
                        drug1_ngrams[term] = float(weight)
        drugs_hash[d] = drug1_ngrams
    return drugs_hash

def compute_pval(sim,background):
    """
    Given a background set and a similarity score, compute a p-value.
    """
    greater = 0
    for back in background:
        if back > sim:
            greater = greater + 1
    return (float(greater)/float(len(background)))

def cosine_sim(drug1_ngrams,drug2_ngrams):
    """
    Quantify the similarity between a pair of drug ppi vectores. 
    """ 
    # sum the weights of drug1 vector
    drug1_bottom = 0.0
    for term in drug1_ngrams:
        drug1_bottom = drug1_bottom + math.pow(drug1_ngrams[term],2.0)
    drug1_bottom = math.sqrt(drug1_bottom)
    
    # sum the weights of drug2 vector
    drug2_bottom = 0.0
    for term in drug2_ngrams:
        drug2_bottom = drug2_bottom + math.pow(drug2_ngrams[term],2.0)
    drug2_bottom = math.sqrt(drug2_bottom)

    # sum the multiplication of the terms in commong
    same_terms = {}
    top = 0.0
    for term in drug1_ngrams:
        if drug2_ngrams.has_key(term):
            top = top + (drug1_ngrams[term]*drug2_ngrams[term])
            same_terms[term] = drug1_ngrams[term]*drug2_ngrams[term]

    if drug1_bottom == 0:
        drug1_bottom = 1

    if drug2_bottom == 0:
        drug2_bottom = 1
    
    # compute the similarity score
    bottom = drug1_bottom * drug2_bottom
    final = top/bottom

    return final

def sort_dict_keys_new(tosort):
    """
    Sort a hash and return in reverse order.
    """
    keys = tosort.keys()
    keys.sort( key = tosort.__getitem__ , reverse = True)
    return keys

def compute_logodds_drugs_against_target(target_name,t,drugs,all_results,sim_type):
    """
    Compute the log-odds of a set of drugs against a target. 
    """
    ppi_odds = {}
    ppi_odds_more = {}
    similarity = {}

    for d in drugs:
        test_drug = d.strip()
        scores = ""
        if sim_type == "3d" or sim_type == "2d" or sim_type == "ppi":
            scores = t.similarity(test_drug)
        elif sim_type == "comb":
            scores = t.similarity_comb(test_drug)
        elif sim_type == "comb3":
            scores = t.similarity_comb3(test_drug)
            
        odd_ppi = scores[0]
        prob_ppi_are = scores[1]
        prob_ppi_arenot = scores[2]
        div_ppi = scores[3]
        hist_ppi1 = scores[4]
        hist_ppi2 = scores[5]
        p_values_ppi1 = scores[6]
        p_values_ppi2 = scores[7]

        ppi_odds[test_drug] = odd_ppi
        ppi_odds_more[test_drug] = [odd_ppi,prob_ppi_are,prob_ppi_arenot,div_ppi,hist_ppi1,hist_ppi2,p_values_ppi1,p_values_ppi2]

    # sort the scores
    ppi_odds_sorted = sort_dict_keys_new(ppi_odds)
    for i in ppi_odds_sorted:
        odd_ppi_more = ppi_odds_more[i]
        odd_ppi_hist = odd_ppi_more[4]
        st_ppi = str(odd_ppi_hist[0])+" "+str(odd_ppi_hist[1])+" "+str(odd_ppi_hist[2])+" "+str(odd_ppi_hist[3])+" "+str(odd_ppi_hist[4])+" "+str(odd_ppi_hist[5])
        odd_ppi_hist_not = odd_ppi_more[5]
        st_ppi_not = str(odd_ppi_hist_not[0])+" "+str(odd_ppi_hist_not[1])+" "+str(odd_ppi_hist_not[2])+" "+str(odd_ppi_hist_not[3])+" "+str(odd_ppi_hist_not[4])+" "+str(odd_ppi_hist_not[5])

        st_all = ""
        if t.drugs_all_hash.has_key(i):
            if t.drugs_all_hash[i] == "primary":
                st_all = target_name+","+i+","+str(round(odd_ppi_more[0],2))+",1\n"
            elif t.drugs_all_hash[i] == "secondary":
                st_all = target_name+","+i+","+str(round(odd_ppi_more[0],2))+",2\n"
        else:
            st_all = target_name+","+i+","+str(round(odd_ppi_more[0],2))+",0\n"
        all_results.write(st_all)

def read_all_scores_sims(all_scores_file,sim_type):
    """
    Read the similarity scores to be used in the log-odds computations.
    
    """ 
    all_scores = {}
    for line in all_scores_file:
        tokens = line.split(",")
        if not tokens[0] == "drug 1":
            if sim_type == "3d":
                key1 = tokens[0]+" "+tokens[1]
                all_scores[key1] = [float(tokens[2]),float(tokens[3].strip())]

                key2 = tokens[1]+" "+tokens[0]
                all_scores[key2] = [float(tokens[2]),float(tokens[3].strip())]
            elif sim_type == "2d":
                key1 = tokens[0]+" "+tokens[1]
                all_scores[key1] = [float(tokens[4]),float(tokens[5].strip())]

                key2 = tokens[1]+" "+tokens[0]
                all_scores[key2] = [float(tokens[4]),float(tokens[5].strip())]
            
            elif sim_type == "ppi":
                key1 = tokens[0]+" "+tokens[1]
                all_scores[key1] = [float(tokens[6]),float(tokens[7].strip())]

                key2 = tokens[1]+" "+tokens[0]
                all_scores[key2] = [float(tokens[6]),float(tokens[7].strip())]
            
            elif sim_type == "3d-2d":
                key1 = tokens[0]+" "+tokens[1]
                all_scores[key1] = [float(tokens[2]),float(tokens[3].strip()),float(tokens[4]),float(tokens[5].strip())]

                key2 = tokens[1]+" "+tokens[0]
                all_scores[key2] = [float(tokens[2]),float(tokens[3].strip()),float(tokens[4]),float(tokens[5].strip())]
            
            elif sim_type == "3d-ppi":
                key1 = tokens[0]+" "+tokens[1]
                all_scores[key1] = [float(tokens[2]),float(tokens[3].strip()),float(tokens[6]),float(tokens[7].strip())]

                key2 = tokens[1]+" "+tokens[0]
                all_scores[key2] = [float(tokens[2]),float(tokens[3].strip()),float(tokens[6]),float(tokens[7].strip())]
                
            elif sim_type == "2d-ppi":
                key1 = tokens[0]+" "+tokens[1]
                all_scores[key1] = [float(tokens[6]),float(tokens[7].strip()),float(tokens[4]),float(tokens[5].strip())]

                key2 = tokens[1]+" "+tokens[0]
                all_scores[key2] = [float(tokens[6]),float(tokens[7].strip()),float(tokens[4]),float(tokens[5].strip())]
                
            elif sim_type == "3d-2d-ppi":
                key1 = tokens[0]+" "+tokens[1]
                all_scores[key1] = [float(tokens[2]),float(tokens[3].strip()),float(tokens[4]),float(tokens[5]),float(tokens[6]),float(tokens[7].strip())]

                key2 = tokens[1]+" "+tokens[0]
                all_scores[key2] = [float(tokens[2]),float(tokens[3].strip()),float(tokens[4]),float(tokens[5]),float(tokens[6]),float(tokens[7].strip())]

    return all_scores

"""
Read the drugs hash
"""
def read_drug_names_hash(drugs_file):
    drugs = {}
    for line in drugs_file:
        drugname = line.strip()
        drugs[drugname] = 1
    return drugs
    
def read_logodds_file(logodds_filename):
    logodds_file = open(logodds_filename)
    logodds = {}
    for line in logodds_file:
        tokens = line.split(",")
        target = tokens[0]
        drug = tokens[1]
        logodds_score = tokens[2].strip()
        key = target+" "+drug
        logodds[key] = logodds_score
        
    logodds_file.close()
    
    return logodds
    
if __name__ == "__main__":
    if sys.argv[1] == "-generate-3d-similarity-files":
        """
        Generate the all-by-all 3D similarity files of the 602 drugs (because sim molecule x and y is same as molecule y and x, only x y was computed). 

        CMD:
        ./yera_jmc_2013.py -generate-3d-similarity-files Drugs602withPPI /home/eyera/Projects/Yera-JMC-2013/3dsim/structures/pre-final/pre- /home/eyera/Projects/Yera-JMC-2013/3dsim/cmds/
        
        INPUT:
        Drugs602withPPI = name of 602 drugs for all by all computation
        /home/eyera/Projects/Yera-JMC-2013/3dsim/structures/pre-final/pre- = path to pre-searched library with "pre-" appended
        /home/eyera/Projects/Yera-JMC-2013/3dsim/cmds/ = path for location of 3d similarity commands.
        
        OUTPUT:
        /home/eyera/Projects/Yera-JMC-2013/3dsim/cmds/"drugname".self = pre-searched drug file
        /home/eyera/Projects/Yera-JMC-2013/3dsim/cmds/"drugname".others = pre-searched drug file with the other drugs
        
        Execute the following on jaincluster2.ucsf.edu
        parallel /home/eyera/Surflex-v2704/bin/surflex-sim-v2704-cluster64.exe -lscreenopt logodds /home/eyera/Projects/Yera-JMC-2013/3dsim/cmds/{1}.self /home/eyera/Projects/3D-similarity-all-drugs/3dsim/cmds/{1}.others /home/eyera/Projects/Yera-JMC-2013/3dsim/cmds/{1}.logodds :::: Drugs602withPPI

        """
        # parameters to set
        pre_path = sys.argv[3]
        file_outpath = sys.argv[4]
        
        drugs_file = open(sys.argv[2])
        # read the drug_names
        drugs_list = []
        drugs_filenames = []
        for line in drugs_file:
            drugname = line.strip()
            drugs_list.append(drugname)
            drugs_filenames.append(pre_path+drugname+".mol2")
        # generate similarity file
        for i in range(0,len(drugs_list)):
            i_out_filename = file_outpath+drugs_list[i]+".self"
            i_out_file = open(i_out_filename,"w")
            i_out_file.write(drugs_filenames[i]+"\n")
            i_out_file.close()

            j_out_filename = file_outpath+drugs_list[i]+".others"
            j_out_file = open(j_out_filename,"w")
            for j in range(i,len(drugs_list)):
                j_out_file.write(drugs_filenames[j]+"\n")
            j_out_file.close()

    elif sys.argv[1] == "-generate-similarity-files-2d":
        """
        Generate the all-by-all 2D similarity files of the 602 drugs (because sim molecule x and y is same as molecule y and x, only x y was computed). 
        
        CMD:
		./yera_jmc_2013.py -generate-similarity-files-2d Drugs602withPPI /home/eyera/Projects/Yera-JMC-2013/3dsim/structures/min-ring-final/min- /home/eyera/Projects/Yera-JMC-2013/gsim/cmds/
        
        INPUT:
        Drugs602withPPI = name of 602 drugs for all by all computation
        /home/eyera/Projects/Yera-JMC-2013/3dsim/structures/min-ring-final/min- = path to minimized library with "min-" appended
        /home/eyera/Projects/Yera-JMC-2013/gsim/cmds/ = path for location of gsim similarity commands.
        
        OUTPUT:
        /home/eyera/Projects/Yera-JMC-2013/gsim/cmds/"drugname".self = min drug file
        /home/eyera/Projects/Yera-JMC-2013/gsim/cmds/"drugname".others = min drug file with the other drugs
        
        Execute the following on jaincluster2.ucsf.edu
        parallel /home/eyera/Surflex-v2704/bin/surflex-sim-v2704-cluster64.exe gsimlogodds /home/eyera/Projects/Yera-JMC-2013/gsim/cmds/{1}.self /home/eyera/Projects/Yera-JMC-2013/gsim/cmds/{1}.others gsim/ListZINC /home/eyera/Projects/Yera-JMC-2013/gsim/cmds/{1}.glogodds :::: Drugs602withPPI
        
        """
        min_ring_path = sys.argv[3]
        file_outpath = sys.argv[4]
        drugs_file = open(sys.argv[2])
        
        # read the drug_names
        drugs_list = []
        drugs_filenames = []
        for line in drugs_file:
            drugname = line.strip()
            drugs_list.append(drugname)
            drugs_filenames.append(min_ring_path+drugname+"-ring.mol2")

        for i in range(0,len(drugs_list)):
            i_out_filename = file_outpath+drugs_list[i]+".self"
            i_out_file = open(i_out_filename,"w")
            i_out_file.write(drugs_filenames[i]+"\n")
            i_out_file.close()

            j_out_filename = file_outpath+drugs_list[i]+".others"
            j_out_file = open(j_out_filename,"w")

            for j in range(i,len(drugs_list)):
                j_out_file.write(drugs_filenames[j]+"\n")
            j_out_file.close()
			
			
    elif sys.argv[1] == "-parse-3d-similarity-files-logodds":			
        """
        Get the 3D similarity p-values of the all-by-all computation
		CMD:
        ./yera_jmc_2013.py -parse-3d-similarity-files-logodds Drugs602withPPI all_sims-3D-FINAL.csv /home/eyera/Projects/Yera-JMC-2013/3dsim/cmds/

        INPUT:
        Drugs602withPPI = drug names
        all_sims-3D-FINAL.csv = file to write similarity scores to
        /home/eyera/Projects/Yera-JMC-2013/3dsim/cmds/ = path to log-odds scores that contain similarity scores
        
        OUTPUT:
        all_sims-3D-FINAL.csv = all-by-all similarity file
        
        
		"""
        cmds_file = open(sys.argv[2])
        all_sim_file = open(sys.argv[3],"w")
        path_logodds = sys.argv[4]
        
        space = re.compile(r'\s+')
        # go over all the log odds files, to get the p-values
        for line in cmds_file:
            drugname = line.strip()
            print drugname
            logodds_file = open(path_logodds+drugname+".logodds")
            for line in logodds_file:
                tokens = space.split(line.strip())
                if len(tokens) == 8:
                    st = drugname+","+tokens[1].replace(":","")+","+tokens[5]+","+tokens[7]
                    all_sim_file.write(st+"\n")
        all_sim_file.close()
        
    elif sys.argv[1] == "-parse-gsim-similarity-files-logodds":			
        """
        Get the 2D similarity p-values of the all-by-all computation
		
        CMD:
        ./yera_jmc_2013.py -parse-gsim-similarity-files-logodds-gsim Drugs602withPPI all_sims-GSIM-FINAL.csv /home/eyera/Projects/Yera-JMC-2013/gsim/cmds/
        
        INPUT:
        Drugs602withPPI = drug names
        all_sims-GSIM-FINAL.csv = file to write similarity scores to
        /home/eyera/Projects/Yera-JMC-2013/gsim/cmds/ = path of similarity scores 
		
        OUTPUT:
        all_sims-GSIM-FINAL.csv = all-by-all similarity file
        
        """
        cmds_file = open(sys.argv[2])
        all_sim_file = open(sys.argv[3],"w")
        path_logodds = sys.argv[3]
        
        space = re.compile(r'\s+')
        # go over all the log odds files, to get the p-values
        for line in cmds_file:
            drugname = line.strip()
            print path_logodds+drugname+".glogodds"
            logodds_file = open(path_logodds+drugname+".glogodds")
            for line in logodds_file:
                tokens = space.split(line.strip())
                if len(tokens) == 6:
                    st = drugname+","+tokens[1].replace(":","")+","+tokens[3]+","+tokens[5]
                    all_sim_file.write(st+"\n")
            
        all_sim_file.close()
        
    elif sys.argv[1] == "-compute-ppi-similarity":
        """
        Compute ppi similarity between all 602 drug.
        
        CMD:
        ./yera_jmc_2013.py -compute-ppi-similarity Drugs602withPPI /home/eyera/Projects/Yera-JMC-2013/ppi/ppi-vectors/ DrugsPPI-602-sims.csv 
        
        INPUT:
        Drugs602withPPI = drug names
        /home/eyera/Projects/Yera-JMC-2013/ppi/ppi-vectors/ = path to ppi vectors
        DrugsPPI-602-sims.csv = similarity file to output
        
        OUTPUT:
        DrugsPPI-602-sims.csv = raw ppi similarity of all the drugs
        
        """
        ppi_vector_path = sys.argv[3] 
        results_file = open(sys.argv[4],"w")

        drugs_file = open(sys.argv[2])
        drugs_hash = {}
        for line in drugs_file:
            drugs_hash[line.strip()] = 1

        file_ext = ".meddra.mesh.csv"
        ngram_size = 5
        # read all drugs meddra and mesh terms with google weights
        drug_ppi = read_drug_ppi(drugs_hash,ngram_size,file_ext,ppi_vector_path,{})

        print "len(drugs_hash)",len(drugs_hash)
        print "len(drug_ppi)",len(drug_ppi)

        drug_ppi_list = []
        for d in drug_ppi:
            drug_ppi_list.append(d)

        for i in range(0,len(drug_ppi_list)):
            drug_i = drug_ppi_list[i]
            allgram_hash1 = drug_ppi[drug_i]

            print i,drug_i
            for j in  range(i,len(drug_ppi_list)):
                drug_j = drug_ppi_list[j]
                if drug_i != drug_j:
                    allgram_hash2 = drug_ppi[drug_j]
                    # compute similarity
                    score = cosine_sim(allgram_hash1,allgram_hash2)
                    st = drug_i.split(".")[0]+","+drug_j.split(".")[0]+","+str(score)
                    results_file.write(st+"\n")

        results_file.close()
        
    elif sys.argv[1] == "-compute-ppi-pvalues":
        """
        DESCRIPTION:
        Normalize raw ppi scores to p-values by using a ppi background distribution (3D & 2D > 0.5).

        CMD:
        ./yera_jmc_2013.py -compute-ppi-pvalues ppi/602-low-3d-2d-background-1000.csv.dist DrugsPPI-602-sims.csv 602-all_sims-PPI-FINAL.csv
        
        DATA:
        ppi/602-low-3d-2d-background-1000.csv.dist = background distribution
        DrugsPPI-602-sims.csv = raw ppi similariy scores
        602-all_sims-PPI-FINAL.csv = outfile of for final ppi similarity scores and p-values
        
        OUTPUT:
        602-all_sims-PPI-FINAL.csv = final ppi similarity scores and p-values
        
        """

        # get background ppi scores
        background_dist_file = open(sys.argv[2])
        background = []
        for line in background_dist_file:
            sim = float(line.strip())
            background.append(sim)


        # read the PPI similarity scores
        ppi_similarity_filename = sys.argv[3]
        ppi_similarity_file = open(ppi_similarity_filename)
        
        # write out results
        out_file = open(sys.argv[4],"w")
        for line in ppi_similarity_file:
            tokens = line.split(",")
            sim = float(tokens[2])
            pval = compute_pval(sim,background)
            st = line.strip()+","+str(pval)
            print st
            
    elif sys.argv[1] == "-combine-3d-2d-ppi":
        """
        Combine the 2D & 3D & PPI similarity
        
        CMD:
        ./yera_jmc_2013.py -combine-3d-2d-ppi Drugs602withPPI 602-all_sims-3D-FINAL.csv 602-all_sims-GSIM-FINAL.csv 602-all_sims-PPI-FINAL.csv 602-3D-GSIM-PPI-FINAL.csv
        
        INPUT:
        Drugs602withPPI = drug names
        602-all_sims-3D-FINAL.csv = 3d similarity
        602-all_sims-GSIM-FINAL.csv = 2d similarity
        602-all_sims-PPI-FINAL.csv = ppi similarity
        602-3D-GSIM-PPI-FINAL.csv = file to output combined file
        
        OUTPUT:
        602-3D-GSIM-PPI-FINAL.csv = combined file

        """
        drugs_file = open(sys.argv[2])
        drugs_list = []
        for line in drugs_file:
            drug = line.strip()
            drugs_list.append(drug)

        threed_file = open(sys.argv[3])
        threed = {}
        for line in threed_file:
            tokens = line.strip().split(",")
            d1 = tokens[0]
            d2 = tokens[1]
            sim = tokens[2]
            pval = tokens[3]
            threed[d1+" "+d2] = [sim,pval]
            threed[d2+" "+d1] = [sim,pval]
        print "len(threed)",len(threed)
        twod_file = open(sys.argv[4])
        twod = {}
        for line in twod_file:
            tokens = line.strip().split(",")
            d1 = tokens[0]
            d2 = tokens[1]
            sim = tokens[2]
            pval = tokens[3]
            twod[d1+" "+d2] = [sim,pval]
            twod[d2+" "+d1] = [sim,pval]
        print "len(twod)",len(twod)

        ppi_file = open(sys.argv[5])
        ppi = {}
        for line in ppi_file:
            tokens = line.strip().split(",")
            d1 = tokens[0]
            d2 = tokens[1]
            sim = tokens[2]
            pval = tokens[3]
            ppi[d1+" "+d2] = [sim,pval]
            ppi[d2+" "+d1] = [sim,pval]
        print "len(ppi)",len(twod)
        
        out_file = open(sys.argv[6],"w")
        out_file.write("drug 1,drug 2,3d,3d-pval,2d,2d-pval,ppi,ppi-pval\n")
        # unique
        for i in range(0,len(drugs_list)):
            d1 = drugs_list[i]
            for j in range(i,len(drugs_list)):
                d2 = drugs_list[j]
                key = d1+" "+d2
                if d1 != d2:
                    st = d1+","+d2+","+threed[key][0]+","+threed[key][1]+","+twod[key][0]+","+twod[key][1]+","+ppi[key][0]+","+ppi[key][1]
                    out_file.write(st+"\n")
        out_file.close()
        
    elif sys.argv[1] == "-log-odds":
        """
        Compute the log-odds of Given each target that has 5 drugs or more, write out there odds to a file to all the drugs we use.
        
        CMDS:
        3D:
        ./yera_jmc_2013.py -log-odds log-odds/targets_final-602.txt 602-3D-GSIM-PPI-FINAL.csv Drugs602withPPI log-odds/3D_logodds.csv log-odds/targets_final-602/ 3d
        
        2D:
        ./yera_jmc_2013.py -log-odds log-odds/targets_final-602.txt 602-3D-GSIM-PPI-FINAL.csv Drugs602withPPI log-odds/2D_logodds.csv log-odds/targets_final-602/ 2d
        
        PPI:
        ./yera_jmc_2013.py -log-odds log-odds/targets_final-602.txt 602-3D-GSIM-PPI-FINAL.csv Drugs602withPPI log-odds/PPI_logodds.csv log-odds/targets_final-602/ ppi
        
        3D+2D:
        ./yera_jmc_2013.py -log-odds log-odds/targets_final-602.txt 602-3D-GSIM-PPI-FINAL.csv Drugs602withPPI log-odds/3D_2D_logodds.csv log-odds/targets_final-602/ 3d-2d
        
        3D+PPI:
        ./yera_jmc_2013.py -log-odds log-odds/targets_final-602.txt 602-3D-GSIM-PPI-FINAL.csv Drugs602withPPI log-odds/3D_PPI_logodds.csv log-odds/targets_final-602/ 3d-ppi
        
        2D+PPI:
        ./yera_jmc_2013.py -log-odds log-odds/targets_final-602.txt 602-3D-GSIM-PPI-FINAL.csv Drugs602withPPI log-odds/2D_PPI_logodds.csv log-odds/targets_final-602/ 2d-ppi
        
        3D+2D+PPI:
        ./yera_jmc_2013.py -log-odds log-odds/targets_final-602.txt 602-3D-GSIM-PPI-FINAL.csv Drugs602withPPI log-odds/3D_2D_PPI_logodds.csv log-odds/targets_final-602/ 3d-2d-ppi
        
        INPUT: 
        log-odds/targets_final-602.txt = all of the targets for which we have drug annotations
        602-3D-GSIM-PPI-FINAL.csv = 3D,2D, and PPI similarity file
        Drugs602withPPI = the drugs to compute the log odds for
        log-odds/3D_logodds.csv = file to write results to
        log-odds/targets_final-602/ = path to target annotation files
        3d = similarity to use which can be "3d","2d","ppi","3d-2d","3d-ppi","2d-ppi", and "3d-2d-ppi"
        
        OUTPUT:
        log-odds/3D_logodds.csv = the log odds of all the drugs against all the targets
        
        """
        sim_type = sys.argv[7]
        targets_file = open(sys.argv[2])
        all_scores_file = open(sys.argv[3])
        all_scores = read_all_scores_sims(all_scores_file,sim_type)
        drugs = read_drug_names_hash(open(sys.argv[4]))
        odds_all_results_filename = sys.argv[5]
        all_results = open(odds_all_results_filename,"w")
        target_dir = sys.argv[6]
        
        print "sim_type",sim_type
        print "len(all_scores)",len(all_scores)
        print "len(drugs)",len(drugs)
        
        if sim_type == "3d" or sim_type == "2d" or sim_type == "ppi":
            sim_type_target = sim_type
        elif sim_type == "3d-2d" or sim_type == "3d-ppi" or sim_type == "2d-ppi":
            sim_type_target = "comb"
        elif sim_type == "3d-2d-ppi":
            sim_type_target = "comb3"
        else:
            print "Wrong type:",sim_type
            sys.exit()
            
        # for each target compute the log-odds of each molecule
        for line in targets_file:
            tokens = line.split(".csv")
            target_name = tokens[0]
            target_filename = target_dir+line.strip()
            target_file = open(target_filename)
            t = Target(target_name,target_file,drugs,all_scores)
            
            # make sure they are of a specific length
            if not len(t.drugs_all) >= 5:
                continue
            else:
                st = target_name+","+str(len(t.drugs_all))
                print st
            
            compute_logodds_drugs_against_target(target_name,t,drugs,all_results,sim_type_target)

        all_results.close()
    
    elif sys.argv[1] == "-combine-logodds":
        """
        Combine the log-odds into a single file.
        
        CMD:
        ./yera_jmc_2013.py -combine-logodds log-odds/3D_logodds.csv log-odds/2D_logodds.csv log-odds/PPI_logodds.csv log-odds/3D_2D_logodds.csv log-odds/3D_PPI_logodds.csv log-odds/2D_PPI_logodds.csv log-odds/3D_2D_PPI_logodds.csv log-odds/ALL_logodds.csv chembl_380_drug_target_interactions_not_in_spdb.csv
        
        INPUT:
        log-odds/3D_logodds.csv 
        log-odds/2D_logodds.csv 
        log-odds/PPI_logodds.csv 
        log-odds/3D_2D_logodds.csv 
        log-odds/3D_PPI_logodds.csv 
        log-odds/2D_PPI_logodds.csv 
        log-odds/3D_2D_PPI_logodds.csv 
        log-odds/ALL_logodds.csv = path to combined file
        chembl_380_drug_target_interactions_not_in_spdb.csv = new chembl annotations
        
        OUTPUT:
        log-odds/ALL_logodds.csv = all log odds including a "3" in last column for new chembl annotations
        """
        
        threed_logodds_filename = sys.argv[2]
        threed_logodds = read_logodds_file(threed_logodds_filename)
         
        twod_logodds_filename = sys.argv[3]
        twod_logodds = read_logodds_file(twod_logodds_filename)
        
        ppi_logodds_filename = sys.argv[4]
        ppi_logodds = read_logodds_file(ppi_logodds_filename)
        
        threed_twod_logodds_filename = sys.argv[5]
        threed_twod_logodds = read_logodds_file(threed_twod_logodds_filename)
        
        threed_ppi_logodds_filename = sys.argv[6]
        threed_ppi_logodds = read_logodds_file(threed_ppi_logodds_filename)
        
        twod_ppi_logodds_filename = sys.argv[7]
        twod_ppi_logodds = read_logodds_file(twod_ppi_logodds_filename)
        
        threed_twod_ppi_logodds_filename = sys.argv[8]
        threed_twod_ppi_logodds = read_logodds_file(threed_twod_ppi_logodds_filename)
        
        chembl_annotations_file = open(sys.argv[10])
        chembl_annotations_file.next() # go over header
        chembl = {}
        for line in chembl_annotations_file:
            tokens = line.split(",")
            target = tokens[3]
            drug = tokens[4]
            key = target+";"+drug
            chembl[key] = 1
            
        out_file = open(sys.argv[9],"w")
        st = "Target,Drug,3D Logg Odds,2D Log Odds,PPI Log Odds,3D+2D Log Odds,3D+PPI Log Odds,2D+PPI Log Odds,3D+2D+PPI Log Odds,relation"
        out_file.write(st+"\n")
        threed_logodds_file = open(threed_logodds_filename)
        for line in threed_logodds_file:
            tokens = line.split(",")
            target = tokens[0]
            drug = tokens[1]
            logodds = tokens[2]
            type = tokens[3]
            key = target+" "+drug
            threed_log = threed_logodds[key]
            twod_log = twod_logodds[key]
            ppi_log = ppi_logodds[key]
            threed_twod_log = threed_twod_logodds[key]
            threed_ppi_log = threed_ppi_logodds[key]
            twod_ppi_log = twod_ppi_logodds[key]
            threed_twod_ppi_log = threed_twod_ppi_logodds[key]
            
            key = target+";"+drug
            st = ""
            if chembl.has_key(key):
                st = target+","+drug+","+threed_log+","+twod_log+","+ppi_log+","+threed_twod_log+","+threed_ppi_log+","+twod_ppi_log+","+threed_twod_ppi_log+",3"
            else:
                st = target+","+drug+","+threed_log+","+twod_log+","+ppi_log+","+threed_twod_log+","+threed_ppi_log+","+twod_ppi_log+","+threed_twod_ppi_log+","+type.strip()
            out_file.write(st+"\n")
        out_file.close()
        
